@@ -12,7 +12,6 @@
 #'  
 #' @param Y abundance table (matrix or dataframe-like), ideally, 
 #' with names for rows and columns. 
-#' BEWARE: all rows and columns should have positive sums!
 #' @param updateN2 logical, default \code{TRUE}.
 #' If \code{FALSE} the marginal sums are proportional to 
 #' the N2-marginals of the initial table, but the N2-marginals of the returned 
@@ -23,22 +22,27 @@
 #' informative species.
 #' If \code{N2N_N2_species = FALSE},								
 #' the returned transformed table has N2 columns marginals, 
-#' \emph{i.e.} \code{colSums(Y2) = const*N2species(Y2)} with \code{Y2} 
-#' the return value of \code{ipf2N2} and \code{const} a constant.
+#' \emph{i.e.} \code{colSums(Y2) = N2species(Y2)} with \code{Y2} 
+#' the return value of \code{ipf2N2}.
 #' If converged, N2 row marginals are equal to the row sums, \emph{i.e.} 
-#' \code{rowSums(Y2) = approx. N2sites(Y2)}.
-#' @param N2N_N2_species Set marginals proportional to \code{N2(1-N2/N)} Default
-#' \code{TRUE}.
-#' @param N2N_N2_sites Default \code{FALSE}. Do not change.
+#' \code{rowSums(Y2) = approx. const*N2sites(Y2)}and \code{const} a constant.
+#' @param N2N_N2_species Set species marginal to
+#'  the value of \code{N2(1-N2/N)} for each species.
+#' Default \code{TRUE}. If \code{FALSE}, the marginal is set to the \code{N2}
+#' value of each species.
+#' @param N2N_N2_sites Default \code{FALSE} sets the marginal proportional
+#' to the \code{N2} value of each site. 
+#' If \code{TRUE}, the marginal is set to \code{N2(1-N2/m)}, with \code{m} the 
+#' number of species.				 
 #' @param max_iter maximum number of iterative proportional fitting (ipf) 
 #' iterations.
-#' If \code{max_iter == 0}, the columns are divided by their 
-#' informativeness (\code{N2}) or \code{N2(1-N2/N)}) without
-#' further pre-processing. The row sums are sums of 
+#' If \code{max_iter = 0}, the columns are divided by their effective number or
+#' informativeness (\code{N2} or \code{N2(1-N2/N)}, depending on the setting
+#' of \code{N2N_N2_species}) without
+#' further pre-processing and the row sums are then,
+#' with \code{N2N_N2_species = TRUE}, sums of
 #' informativeness instead of effective number of informative species.
 #'
-#' @param crit stopping criterion.
-#' 
 #' @return a matrix of the same order as the input \code{Y},
 #' obtained after ipf to N2-marginals.
 #' 
@@ -52,6 +56,9 @@
 #' in more than halve the number of sites are down-weighted, so that
 #' the row sum is no longer equal to the richness of the site (the number of species),
 #' but proportional to the number of informative species. 
+#' The returned matrix has the intended species marginal (column sums),
+#' by construction of the algorithm, even without convergence. 
+#' On convergence, it has the intended site marginal (row sums).
 #' 
 #' @references
 #' ter Braak, C.J.F. (2019). 
@@ -60,60 +67,79 @@
 #' Methods in Ecology and Evolution, 10 (11), 1962-1971. 
 #' \doi{10.1111/2041-210X.13278}
 #' 
+#' ter Braak, C.J.F. (2026).
+#' Fourth-corner latent variable models overstate confidence in
+#' trait–environment relationships and what to use instead
+#' Environmental and Ecological Statistics.
+#'\doi{10.1007/s10651-025-00696-0}
+#' 
 #' @example demo/dune_ipf2N2.R
 #' 
 #' @export
 ipf2N2 <- function(Y, 
-                   max_iter = 1000, 
+                   max_iter = 10000, 
                    updateN2 = TRUE, 
                    N2N_N2_species = TRUE, 
-                   N2N_N2_sites = FALSE, 
-                   crit = 1.0e-2) {
-  if (N2N_N2_sites) {
-    warning("Setting N2N_N2_sites to TRUE is not recommended.\n")
-  }
+                   N2N_N2_sites = FALSE) {
   Y <- as.matrix(Y)
   rownams <- rownames(Y)
   colnams <- colnames(Y)
   R <- rowSums(Y)
   K <- colSums(Y)
   if (any(R == 0)) {
-    stop("Some sites do not have species.\n", names(R)[which(R == 0)], "\n")
+    warning("Some sites do not have species.\n ",
+            paste(names(R)[which(R == 0)], collapse = " "), "\n")
   }
   if (any(K == 0)) {
-    stop("Some species are absent in every site.\n", 
-         names(K)[which(K == 0)], "\n")
+    warning("Some species are absent in every site.\n", 
+            paste(names(K)[which(K == 0)],  collapse = " "), "\n")
   }
+  K[K < .Machine$double.eps] <- .Machine$double.eps
+  R[R < .Machine$double.eps] <- .Machine$double.eps
   N2spp <- N2spp0 <- fN2N_N2(Y, 2, N2N_N2 = N2N_N2_species)
   N2sites <- N2sites0 <- fN2N_N2(Y, 1, N2N_N2 = N2N_N2_sites)
   N <- nrow(Y)
-  tot <- sum(R)
+  crit0  <- 1.0e10
+  crit1 <- crit0 - 1
   iter <- 0
-  # the criterion is on the weights made relative (what counts in CA)
-  while (iter == 0 || 
-         (max(N2sites / sum(N2sites) - R / sum(R)) > crit / N && iter < max_iter)) { 
-    Y <- Y * (N2sites / R)
-    K <- colSums(Y)
-    K[K < .Machine$double.eps] <- .Machine$double.eps		 
-    if (updateN2) N2spp <- fN2N_N2(Y, 2, N2N_N2 = N2N_N2_species)
+  ratio <- 1
+  if(max_iter == 0){
     Y <- Y %*% diag(N2spp / K)
-    if (max_iter == 0) break
-    iter <- iter + 1
-    R <- rowSums(Y)
-    R[R< .Machine$double.eps] <- .Machine$double.eps												
-    if (updateN2) N2sites <- fN2N_N2(Y, 1, N2N_N2 = N2N_N2_sites)
+  } else {
+    while (crit1 < crit0 && iter < max_iter) { 
+      iter <- iter + 1 
+      crit0 <- crit1
+      Y0 <- Y
+      Y <- Y * ((N2sites / R) * ratio)
+      K <- colSums(Y)
+      if (updateN2) N2spp <- fN2N_N2(Y, 2, N2N_N2 = N2N_N2_species)
+      Y <- Y %*% diag(N2spp / K)
+      R <- rowSums(Y)
+      if (updateN2) N2sites <- fN2N_N2(Y, 1, N2N_N2 = N2N_N2_sites)
+      ratio <- sum(N2spp)/sum(N2sites)
+      mm <- range((ratio * N2sites+1) / (R + 1))
+      crit1 <- max(c(1/mm[1], mm[2]))
+    }
+    Y <- Y0
+    if (updateN2) {
+      N2sites <- fN2N_N2(Y, 1, N2N_N2 = N2N_N2_sites)
+      N2spp <- fN2N_N2(Y, 2, N2N_N2 = N2N_N2_species)
+    }
+    if (iter == max_iter && max_iter > 0) {
+      warning(paste0("No convergence in ", max_iter, " iterations."),"\n")
+    }
+    if (any(Y < 0)){
+      warning("some values in preprocessed Y negative")
+      Y[Y < 0] <- 0 # tiny non-negative values should not occur
+    }
   }
-  if (iter == max_iter && max_iter > 0) {
-    warning(paste0("No convergence in ", max_iter, " iterations."),"\n")
-  }
-  Y[Y < 0] <- 0 # tiny non-negative values do occur in practice														
-  Y <- Y * (tot / sum(Y))
   attr(Y, which = "N2species_original") <- N2spp0
   attr(Y, which = "N2sites_original") <- N2sites0
   attr(Y, which = "N2species") <- N2spp
   attr(Y, which = "N2sites") <- N2sites
   attr(Y, which = "R/N2") <- (R / N2sites) / (sum(R) / sum(N2sites))
   attr(Y, which = "iter") <- iter
+  attr(Y, which = "crit") <- crit0
   colnames(Y) <- colnams
   rownames(Y) <- rownams
   R <- rowSums(Y)
@@ -121,6 +147,16 @@ ipf2N2 <- function(Y,
     message("Warning: unbalanced site totals in return value:",
             "N2 of row sums less than halve the number of rows.\n")
   }
+  if (any(N2sites <= .Machine$double.eps)) {
+    warning("After processing: Some sites do not have species.\n ",
+            paste(rownames(Y)[which(N2sites <= .Machine$double.eps)], 
+                  collapse = " "), "\n")
+  }
+  if (any(N2spp <= .Machine$double.eps)) {
+    warning("After processing: Some species are absent in every site.\n ",
+            paste(colnames(Y)[which(N2spp <= .Machine$double.eps)],  
+                  collapse = " "), "\n")
+  }
   return(Y)
 }
 
@@ -146,7 +182,11 @@ ipf2N2 <- function(Y,
 # @noRd
 # @keywords internal
 fN2 <- function(x) {
-  x <- x / sum(x)
+  sx <- sum(x)
+  if (is.na(sx) || sx < 1.0e-6) {
+    return(.Machine$double.eps)
+  }
+  x <- x / sx		 
   return(1 / sum(x * x))
 }
 
@@ -156,9 +196,7 @@ fN2N_N2 <- function(Y,
                     margin,
                     N2N_N2 = TRUE) {
   N2 <- apply(X = Y, MARGIN = margin, FUN = fN2)
-  # the reason for the 2 is to strech the number to N/2 when N2 = N/2
   margin1 <- if (margin == 1) 2 else 1
-  if (N2N_N2) N2 <- N2 * (1 - N2 / dim(Y)[margin1])
-  N2[is.na(N2)] <- 0
+  if (N2N_N2) N2 <- N2 * pmax(1 - N2 / dim(Y)[margin1], .Machine$double.eps)
   return(N2)
 }
\ No newline at end of file
